#!/bin/bash

# ---- effect processor script v 1.2 ----

if [ "${INCDIR}" == "" ] ; then
	INCDIR="/Csound/Include" ;	# include directory
fi

# temp file names

TMPORCNAME="/tmp/effectproc.tmp.$$.orc"
TMPSCONAME="/tmp/effectproc.tmp.$$.sco"
TMPFILNAME="/tmp/effectproc.tmp.$$.sf"

# remove old temp files

rm -f "${TMPORCNAME}" "${TMPSCONAME}" "${TMPFILNAME}" score.srt

PRG_NAME=`basename "${0}"`

function	PrintUsage ()
{
	echo -n "Usage: ${PRG_NAME} <infile> <outfile> <conf.file> "
		echo "<scorefile>        \\"
	echo "                        [-r reverb_file] [-s stereo_infile]"
	echo "infile, outfile: name of input, and output file"
	echo -n "                 if outfile is dac then output is sent to "
		echo "sound device"
	echo "conf.file:       configuration file"
	echo "scorefile:       score file storing effect parameters"
	echo "                 (example files for conf.file and scorefile are"
	echo "                  ${INCDIR}/Configure.effect.orc,"
	echo "                  ${INCDIR}/Parameters.effect.sco)"
	echo -n "reverb_file:     convolve file for reverberation "
		echo "(default: Reverb.cv)"
	echo "stereo_infile:   stereo input file if enabled in ftable 22"
	echo ""
	exit ${1} ;
}

# calc_expr <scale> <expression>
# evaluate math expression, and print to stdout

function	calc_expr ()
{
	echo -e " scale=${1} ; ${2} " | bc -l -q | tr -d " \n\t" ;
}

# check_file <filename>
# exit if "filename" does not exist or is not a file

function	check_file ()
{
	if ( ! ( test -e "${1}" ) ) ; then
		echo -e "${PRG_NAME}: ${1}: not found" ; exit -1 ;
	elif ( ! ( test -f "${1}" ) ) ; then
		echo -e "${PRG_NAME}: ${1}: not a file" ; exit -1 ;
	fi ;
}

# ---- get commandline options ----

if ( ( [ $# == 0 ] ) || ( [ $# == 1 ] )					\
     || ( [ $# == 2 ] ) || ( [ $# == 3 ] ) ) ; then	# insufficient args
	PrintUsage -1 ;
fi

INFILENAME="${1}" ; shift		# input file name
OUTFILNAME="${1}" ; shift		# output file name
CONFFLNAME="${1}" ; shift		# conf. file name
SCOREFNAME="${1}" ; shift		# score file name

INFIL2NAME="/dev/zero"	# default stereo input file name
RVBFILNAME="Reverb.cv"	# default reverb file name

while [ $# != 0 ] ; do	# optional parameters
	case "${1}" in
		("-r")	shift ; if [ $# != 0 ] ; then
				RVBFILNAME="${1}" ; shift ;	# reverb file
			else
				PrintUsage -1 ;
			fi				;;
		("-s")	shift ; if [ $# != 0 ] ; then
				INFIL2NAME="${1}" ; shift ;	# stereo file
			else
				PrintUsage -1 ;
			fi				;;
		(*)	PrintUsage -1			;;
	esac ;
done

# ---------------- check input files ----------------

check_file "${INFILENAME}"
check_file "${CONFFLNAME}"
check_file "${SCOREFNAME}"

# input file 1 has to be mono

CHN=`SoundFileChannels "${INFILENAME}" | tr -d " \t\r\n"`
if [ "${CHN}" != "1" ] ; then
	echo -e "${PRG_NAME}: ${INFILENAME}: invalid input file"
	exit -1 ;
fi
SRATE=`SoundFileSampleRate "${INFILENAME}" | tr -d " \t\r\n"`	# sample rate
KSMPS="32"							# ksmps

# input file 2 (stereo)

if [ "${INFIL2NAME}" != "/dev/zero" ] ; then
	check_file "${INFIL2NAME}"
	CHN=`SoundFileChannels "${INFIL2NAME}" | tr -d " \t\r\n"`
	if [ "${CHN}" != "2" ] ; then
		echo -e "${PRG_NAME}: ${INFIL2NAME}: invalid input file"
		exit -1 ;
	fi
	SR2=`SoundFileSampleRate "${INFIL2NAME}" | tr -d " \t\r\n"`
	if [ "${SRATE}" != "${SR2}" ] ; then
		echo -e "${PRG_NAME}: input files with different sample rate"
		exit -1 ;
	fi ;
fi

# reverb file

REVERB=`grep -e "REVERB" < "${CONFFLNAME}" | grep -e "ReverbOn" |	\
	tr -d " \t\r\n"`
if [ "${REVERB}" != "" ] ; then
	check_file "${RVBFILNAME}"
	RVBLENGTH=`TestCvFile2 "${RVBFILNAME}" "${SRATE}" "${KSMPS}"`
	if [ "${RVBLENGTH}" == "" ] ; then
		echo -e "${PRG_NAME}: ${RVBFILNAME}: invalid reverb file"
		exit -1 ;
	fi ;
else
	RVBFILNAME="" ; RVBLENGTH="0" ;		# no reverb
fi

# -------- set up orc and sco files, and render output --------

# orc file

echo -n "m4_changecom" | m4 -P						      \
	-D__SRATE="${SRATE}" -D__KSMPS="${KSMPS}"			      \
	-D__CONF_NAME="\"${CONFFLNAME}\"" -D__INFILENAME="\"${INFILENAME}\""  \
	-D__INFIL2NAME="\"${INFIL2NAME}\"" -D__RVB_FILE="\"${RVBFILNAME}\""   \
	-D__REVERBLENGTH="${RVBLENGTH}"					      \
	- "${INCDIR}/EffectProc-1.2/EffectProc-1.2.orc"	>  "${TMPORCNAME}"

# score file

echo -e "\nt 0 60\n"					>  "${TMPSCONAME}"
cat "${SCOREFNAME}"					>> "${TMPSCONAME}"
echo -e -n "\ni 1 0 1 "					>> "${TMPSCONAME}"
grep -e "^.*#define.*XTime" < "${SCOREFNAME}" |				\
	sed 's/^.*#.*#\(.*\)#.*$/\1/' |					\
	tr -d " \n\r\t" | sed 's/^+//'			>> "${TMPSCONAME}"
echo -e "\n\ne\n"					>> "${TMPSCONAME}"

# render output

if [ "${OUTFILNAME}" == "dac" ] ; then
	csound -d -m0 -H0 -s -h -b 16384 -B 16384 -o devaudio		\
		"${TMPORCNAME}" "${TMPSCONAME}" ;	# realtime output
else
	csound -d -m7 -H1 -f -J -b 65536 -B 65536 -o "${TMPFILNAME}"	\
		"${TMPORCNAME}" "${TMPSCONAME}"		# output to file
	rm -f "${OUTFILNAME}"
	if [ "${REVERB}" != "" ] ; then
		rm -f "${TMPORCNAME}" "${TMPSCONAME}"	# extract output

		echo -n "m4_changecom" | m4 -P				      \
			-D__SRATE="${SRATE}"				      \
			-D__KSMPS="${KSMPS}"				      \
			-D__TMPFILNAME="\"${TMPFILNAME}\""		      \
			-D__REVERBLENGTH="${RVBLENGTH}"			      \
			- "${INCDIR}/EffectProc-1.2/Extract.orc"	      \
							>  "${TMPORCNAME}"

		echo -e "\nt 0 60\n\ni 1 0 1\n\ne\n"	> "${TMPSCONAME}"

		echo -e -n "Extracting output file ... "
		csound -d -m0 -f -J -b 65536 -B 65536 -o "${OUTFILNAME}"   \
			"${TMPORCNAME}" "${TMPSCONAME}" > /dev/null 2>&1
		echo -e "done."

		rm -f "${TMPFILNAME}" ;
	else
		mv -f "${TMPFILNAME}" "${OUTFILNAME}" ;
	fi ;
fi

# remove tmp files

rm -f "${TMPORCNAME}" "${TMPSCONAME}" "${TMPFILNAME}" score.srt

